/***************************************************************************
                          analyzer.c  -  description
                             -------------------
    begin                : Thu Sep 19 2002
    copyright            : (C) 2002 by Jan Fernquist, Florian Boor
    email                : boor@unix-ag.org
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <string.h>
#include "analyzer.h"
#include "prismstumbler.h"
#include "psgui.h"

char logbuf[120];
extern ScanResult_t Res;
int LOG_PACKETS = 0;
int FAKEAP_FILTER = 0;


inline int extract_subnet(unsigned char* adrinfo)
{
	if (adrinfo[0] == 0xFF) return FALSE;
	if (adrinfo[1] == 0xFF) { adrinfo[1]=0x00;adrinfo[2]=0x00;adrinfo[3]=0x00; return TRUE;}
	if (adrinfo[2] == 0xFF) { adrinfo[2]=0x00;adrinfo[3]=0x00; return TRUE;}
	if (adrinfo[3] == 0xFF) { adrinfo[3]=0x00; return TRUE;}
	return FALSE;		
}


int isResolved(const unsigned char *p) {
   unsigned char sum;
   if (p[1] == 255 && p[0] > 2 && p[0] < 16) return 1;
   sum = p[0] + p[1];
   if (sum == 1 && (p[2] <= 0x0A || p[2] == 0xFF)) return 1;
   if (sum <= 0x0C && (p[2] >= 0xF2 && p[2] <= 0xFE && p[2] != 0xFD)) return 1;
   return 0;
}


/*
*
 * Get the different data from the MGMT header. Fill global struct
 */

int processPacket(unsigned char *packet, int len,int RealChannel){
	FixedMgmt_t  *M;
	ProbeFixedMgmt_t  *P;
	AdmInfo_t    *A;
  	unsigned char *iv = NULL;
	unsigned char *admbits,*fixbits,*varBits;
	int tagType,tagLen;
/*	struct servent *service_i1;
	struct servent *service_i2;
	unsigned short int p1,p2;
*/
	//struct?
//	int protocol;
	unsigned char *ip_src, *ip_dest;
	unsigned short *port_src, *port_dest;
	unsigned char *ip_load;

	admbits = packet;
	fixbits = &packet[sizeof(AdmInfo_t)]; /* This is where the payload starts*/

	A = (AdmInfo_t *) admbits; /* First the frame from the card */
	M =  (FixedMgmt_t *) fixbits; /*Then the actual data*/
	P =  (ProbeFixedMgmt_t *) fixbits; /*Then the actual data*/

	memset(&Res,0,sizeof(Res));

	Res.FrameType = M->frametype;


	if( M->frametype  == MGT_PROBE_RESP || M->frametype == MGT_BEACON ){
			Res.isAp   = IS_TO_DS(COOK_FLAGS(M->frametype)) != 0;
			Res.isAp   += IS_FROM_DS(COOK_FLAGS(M->frametype)) != 0; // in this case we have more than one ap
            Res.hasWep = IS_WEP(COOK_FLAGS(M->frametype)) != 0;
			Res.hasWep += IS_WEP_REQUIRED(M->Capabilities) != 0;
	}
	if( M->frametype  == MGT_PROBE || M->frametype == MGT_BEACON ){
			if(! ( M->DestAddr[0] == 0xff && M->DestAddr[1] == 0xff &&
				M->DestAddr[2] == 0xff && M->DestAddr[3] == 0xff &&
				M->DestAddr[4] == 0xff && M->DestAddr[5] == 0xff
			     ))  return 0;
	}

	if( M->frametype  == MGT_PROBE  ){
		 Res.isAp   = AP_PROBE  ;
		 Res.hasWep =  0;
		 Res.Channel = RealChannel ;
	}

	Res.Signal = A->signal.data;
	Res.Noise = A->noise.data;

	if (M->frametype & DATA_PURE) // do we have a data packet?
	{
		int sdt = 0;
		int hdrlen = 0;
		Res.isData = 1;
		Res.Channel = RealChannel;
		Res.subnet[0]=0;
		Res.subnet[1]=0;
		Res.subnet[2]=0;

		// analyze packet
		sdt = (M->frametype & 0x0300) >> 8;
		switch(sdt)
		{
			case 0:  // node to node
				// evtl. bssid = src?
				hdrlen = DATA_SHORT_HDR_LEN;
				sprintf(Res.DestMac,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
					M->DestAddr[0], M->DestAddr[1],
					M->DestAddr[2], M->DestAddr[3],
					M->DestAddr[4], M->DestAddr[5]
				);
				sprintf(Res.SrcMac,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
					M->SrcAddr[0], M->SrcAddr[1],
				 	M->SrcAddr[2], M->SrcAddr[3],
				 	M->SrcAddr[4], M->SrcAddr[5]
				);
				sprintf(Res.BssId,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
					M->BssId[0], M->BssId[1],
				 	M->BssId[2], M->BssId[3],
				 	M->BssId[4], M->BssId[5]
				);
			break;

			case 2: // from node
				hdrlen = DATA_SHORT_HDR_LEN;
				sprintf(Res.DestMac,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
					M->DestAddr[0], M->DestAddr[1],
					M->DestAddr[2], M->DestAddr[3],
					M->DestAddr[4], M->DestAddr[5]
				);
				sprintf(Res.BssId,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
					M->SrcAddr[0], M->SrcAddr[1],
				 	M->SrcAddr[2], M->SrcAddr[3],
				 	M->SrcAddr[4], M->SrcAddr[5]
				);
				sprintf(Res.SrcMac,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
					M->BssId[0], M->BssId[1],
				 	M->BssId[2], M->BssId[3],
				 	M->BssId[4], M->BssId[5]
				);
			break;
			case 1: // from ap
				hdrlen = DATA_SHORT_HDR_LEN;
				sprintf(Res.BssId,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
					M->DestAddr[0], M->DestAddr[1],
					M->DestAddr[2], M->DestAddr[3],
					M->DestAddr[4], M->DestAddr[5]
				);
				sprintf(Res.SrcMac,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
					M->SrcAddr[0], M->SrcAddr[1],
				 	M->SrcAddr[2], M->SrcAddr[3],
				 	M->SrcAddr[4], M->SrcAddr[5]
				);
				sprintf(Res.DestMac,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
					M->BssId[0], M->BssId[1],
				 	M->BssId[2], M->BssId[3],
				 	M->BssId[4], M->BssId[5]
				);
			break;
			case 3: // ap to ap
				hdrlen = DATA_LONG_HDR_LEN;
				break;
				sprintf(Res.DestMac,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
					M->DestAddr[0], M->DestAddr[1],
					M->DestAddr[2], M->DestAddr[3],
					M->DestAddr[4], M->DestAddr[5]
				);
				sprintf(Res.BssId,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
					M->SrcAddr[0], M->SrcAddr[1],
				 	M->SrcAddr[2], M->SrcAddr[3],
				 	M->SrcAddr[4], M->SrcAddr[5]
				);
				sprintf(Res.SrcMac,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
					M->BssId[0], M->BssId[1],
				 	M->BssId[2], M->BssId[3],
				 	M->BssId[4], M->BssId[5]
				);

			//return 0; // we don't know the bssid
		} // case

		// a usable data packet?
		if (len > (hdrlen+10+g_offset))
		{
			iv = packet + hdrlen + g_offset - 8 ;

	        // What about WEP?
			if (((IS_WEP(COOK_FLAGS(M->frametype)) != 0)/* || (*((unsigned short *)iv) != 0xAAAA)*/))
			{
				// we have an encrypted packet with iv
			
				// check if ist is really usable
            	if (isResolved(iv)) {
					// do_something
					Res.hasIntIV = 1;
					sprintf(logbuf,"weak iv: %.2X:%.2X:%.2X:%.2X\n",iv[0],iv[1],iv[2],iv[3]);
					printlog(logbuf);
				}
			}
			else // looks like unencrypted traffic
			{
				if ((iv[IP_OFFSET_PROTO]) == IP_PROTO_TCP) {
						ip_src = iv+12;
						ip_dest = iv+16;
						ip_load = iv+20;//+ 4 * (*((unsigned short *)iv) & 0xF0); // length in byte, masq version
						port_src = (unsigned short *)ip_load;						
						port_dest = (unsigned short *)(ip_load+1);						
						if(ip_dest[3] == 0xFF) {
							if (extract_subnet(ip_dest)){
								sprintf(logbuf,"net from tcp %i.%i.%i.%i \n",ip_dest[0],ip_dest[1],ip_dest[2],ip_dest[3]);
								printlog(logbuf);
								memcpy(Res.subnet,ip_dest,4);
							}
						}
						if (LOG_PACKETS)
						{
							sprintf(logbuf,"TCP from %i.%i.%i.%i to %i.%i.%i.%i\n",ip_src[0],ip_src[1],ip_src[2],ip_src[3],ip_dest[0],ip_dest[1],ip_dest[2],ip_dest[3]);
							printlog(logbuf);
							sprintf(logbuf,"    port %i             %i\n",port_src[0],port_dest[0]);		
							printlog(logbuf);
						}
/*						p1 = port_src[0];  p2 = port_dest[0];
						service_i1 = getservbyport(htons(p1),"tcp");
						service_i2 = getservbyport(htons(p2),"tcp");
						if (service_i1) printf("serv1: %s\n",service_i1->s_name);		
						if (service_i2) printf("serv2: %s\n",service_i2->s_name);		
*/				}

				if ((iv[IP_OFFSET_PROTO]) == IP_PROTO_UDP) {
			
						ip_src = iv+12;
						ip_dest = iv+16;
						ip_load = iv+20;// + 4 * (*((unsigned short *)iv) & 0xF0); // length in byte, masq version
						port_src = (unsigned short *)ip_load;						
						port_dest = (unsigned short *)(ip_load+1);						

						if(ip_dest[3] == 0xFF) {
							if (extract_subnet(ip_dest)){
								memcpy(Res.subnet,ip_dest,4);
								sprintf(logbuf,"net from udp %i.%i.%i.%i \n",Res.subnet[0],Res.subnet[1],Res.subnet[2],Res.subnet[3]);
								printlog(logbuf);
							}
						}

						if (LOG_PACKETS)
						{
							sprintf(logbuf,"UDP from %i.%i.%i.%i to %i.%i.%i.%i\n",ip_src[0],ip_src[1],ip_src[2],ip_src[3],ip_dest[0],ip_dest[1],ip_dest[2],ip_dest[3]);
							printlog(logbuf);
							sprintf(logbuf,"    port %i             %i\n",port_src[0],port_dest[0]);		
							printlog(logbuf);
						}
/*						p1 = port_src[0];  p2 = port_dest[0];
						service_i1 = getservbyport(htons(p1),"udp");
						service_i2 = getservbyport(htons(p2),"udp");
						if (service_i1) printf("serv1: %s\n",service_i1->s_name);		
						if (service_i2) printf("serv2: %s\n",service_i2->s_name);		
*/				}

				if ((iv[IP_OFFSET_PROTO]) == IP_PROTO_ICMP) {
					if (LOG_PACKETS)
					{
						sprintf(logbuf,"ICMP\n");
						printlog(logbuf);
					}
				}

				// dhcp check taken from kismet
				if (memcmp(&iv[DHCPD_OFFSET], DHCPD_SIGNATURE,sizeof(DHCPD_SIGNATURE)) == 0)
				{
					Res.dhcp = 1;
				}
			}
		} //if len

		return 1;
	} //if
    else // handle beacon....
	{
		sprintf(Res.DestMac,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
		M->DestAddr[0], M->DestAddr[1],
		 M->DestAddr[2], M->DestAddr[3],
		 M->DestAddr[4], M->DestAddr[5]
		);
		sprintf(Res.SrcMac,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
			M->SrcAddr[0], M->SrcAddr[1],
		 	M->SrcAddr[2], M->SrcAddr[3],
			M->SrcAddr[4], M->SrcAddr[5]
		);

		sprintf(Res.BssId,"%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
			M->BssId[0], M->BssId[1],
		 	M->BssId[2], M->BssId[3],
		 	M->BssId[4], M->BssId[5]
		);
	} // handle packet types...
	if(  M->frametype == MGT_PROBE ){
		if( len <= sizeof(ProbeFixedMgmt_t) ) return 0;

		varBits = &fixbits[sizeof(ProbeFixedMgmt_t)];

	}else{ /* Beacon and Probe Response */
		if( len <= sizeof(FixedMgmt_t) ) return 0;

		varBits = &fixbits[sizeof(FixedMgmt_t)];
	}

	/* Get the tagged values
	 * There is a type and a length field
	 * Followed by tha actual data
         */
	while( varBits < (packet+len)){

		tagType = varBits[0];
		tagLen = varBits[1];
		varBits+=2;
	
	
		switch(tagType){
			case TAG_SSID:
					strncpy(Res.SSID,varBits,
				    tagLen > sizeof(Res.SSID)?
					sizeof(Res.SSID)-1:tagLen);
					break;

			case TAG_DS_PARAMETER:
						if(tagLen == 1){
							Res.Channel = *varBits;
						}
						break;
			/* Skip all other values for the moment */
			case TAG_SUPP_RATES:
			case TAG_FH_PARAMETER:
			case TAG_CF_PARAMETER:
			case TAG_TIM:
			case TAG_IBSS_PARAMETER :
			case TAG_CHALLENGE_TEXT:	
						 break;
		}	
		varBits+=tagLen;
	}
	return 1;
}
