/* prismstumbler.c - Utility Scan for 802_11 networks 
 * 
 * This utility is written for use with IEEE 802.11 adapters based
 * on Intersil's PRISM II chipset (PCMCIA). 
 * The linux driver for these cards can be found on www.linux-wlan.com
 * It has been verified with a D-LINK DWL-650 WL100 IEEE 802.11 adapter.
 * Created by: Jan Fernquist  Jan.B.Fernquist@telia.com
 * 
 *  The main idea for this program create a NetStumbler like utlity to find W-LANS
 *
 * Base code was from PrismDump.
 * Code also Snarfed from Ethreal and airsnort
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

/*
 * RevHist
 *  
 * When		Who		What
 *
 * 02-10-03 flo     removed analyzer code to analyzer.c, added gpd-fix support
 *
 * 02-06-01	jafer		getPacket() ifdef away and moved to ifac_wlan.c
 *
 * 02-04-03	jafer		Size check of SSID data
 *				Not includeing iface-wlan anymore
 *
 *
 * 02-03-26	jafer		Waits for a Q on stdin for termination
 *				Removed wlan-ng dependant code to iface-wlan.c
 *
 * 02-03-06     jafer		New options -S N  Tune in on Single channel	
 *				-F  F  Save data to file F as well as stdout
 *			        Time Field has a decimal to get difference form packet arriving
 *			   	at the same second. Used in Perlprog to sort the last seen packet firts
 *
 * 02-01-09	jafer		Embedded use of the functtion wlanctl-ng by using 
 *				the actual program
 *
 *
 */

#include <pthread.h>
#include <string.h>
#include "prismstumbler.h"
#include "fxgps.h"
#include "analyzer.h"

#define HELPTEXT "Call with no parameters or with the following options\n\
	-S Channel   - Only listen on a specific Channel (1-14)\n\
	-F FileName  - Save output to an file as well as stdout\n\
	-w FileName  -  Create (and update) Gpsdrive Waypointfile\n\
	-d  Database only mode (scanner inactive)\n\n"

unsigned int stop_sniffing = 0;

int newnet_count = 0;

pthread_mutex_t buf_mutex = PTHREAD_MUTEX_INITIALIZER;

/*
 * How many packet should we read if we have a signal on a channel????
 * After that we force a channel change anyway
 */
#define CHANNEL_READ_COUNT 10

int g_offset = 0;
char *buf;


/*
 * Use globals, ugly but efficient
 */
static int RawSock;
ScanResult_t Res;
int SingleChannel = 0;
static int gps_active = 0;
char * OutFileName = NULL;
FILE * OutFile  = NULL;
char * WPFileName = NULL;
FILE * WPFile  = NULL;
static float Long,Lat;


/*
 * Some forward refs
 */

void closePacket(int);
int openPacket(void);
int selectChannel(int channel);
void shutCard(void);

int getPacket(unsigned char *buf,int maxlen,int timeout) ;


/*
 * Help the poor guy!
 */

void HelpAndBye(void){
	fprintf(stderr,"%s",HELPTEXT);
	exit(1);
}


void psmain (int fixchannel) {
	int recvlen;
	unsigned char msgbuf[MAX_BUFFER_SIZE];
	int Channel=0;
	int MaxPackPerChannel;
	int cupdate = 0;
	time_t now,last_time;
	int Time_Add;

	sleep(2);

	if (fixchannel) SingleChannel = fixchannel;
	
    if (gps_initialize() > 0)
        {
          gps_active=1;
          fprintf(stderr,"GPSd available... trying to use. :-)\n");
          update_gpsdata();
        }
    else
          fprintf(stderr,"Can't find woking GPSd... we'll have fun without it ;-)\n");
					
	setvbuf(stdin,NULL,_IONBF,0);
	setpriority (PRIO_PROCESS, 0, -20);


	if(  (RawSock=openPacket()) < 0 ){
#ifdef PRE_0_14 /* New version is in if_wlan.c */
		perror(" Cant open Netlink Socket");
#else
		perror("Cant open pcap device");
#endif
		pthread_exit(NULL);
	}


	time(&last_time);
	Time_Add = 0	;

	while( stop_sniffing == 0){

		if( ! SingleChannel ){
			Channel++;
			if( Channel > 14) Channel = 1;
		}else{
			Channel = SingleChannel;
		}

		if( selectChannel(Channel) != 0 ) break;

		cupdate++;
		if (cupdate == 50) {
			update_colors();
			cupdate = 0;
		}

	
		 MaxPackPerChannel = CHANNEL_READ_COUNT;
		do{
			memset (msgbuf, 0, MAX_BUFFER_SIZE);
			recvlen = getPacket(msgbuf, MAX_BUFFER_SIZE, 100);
            // do we still know _where_ we are?
  			if (gps_active) {
				if (gps_get_xy(&Lat,&Long) == 0) // no gps fix
					{
						Lat=0.0;
						Long=0.0;
					}
			}

			if( recvlen < 0 ) break;  /* Error */

			if( recvlen ==  0 ) break;

			if(  processPacket(msgbuf, recvlen,Channel)  ){  /* Got A valid inetersting ?*/

				if( Res.Channel < 1 || Res.Channel > 14 ){
						 continue; /* Bogus pkt*/
				}
				time(&now);
				if( now == last_time ){
					Time_Add++;
				}else{
					last_time = now;
					Time_Add = 0;
				}

				if( OutFile ){
					fprintf(OutFile,"%d.%.2d;%s;%d;%d;%d;%d;%d;%s;%s;%s;%.4X;%f;%f\n",
						now,Time_Add,
						Res.SSID,Res.Channel,Res.hasWep,Res.isAp,
						Res.Signal,Res.Noise,
						Res.DestMac,Res.SrcMac,Res.BssId ,Res.FrameType,
						Lat,Long
					);	
				}

				/*
					Now transfer collected data...
				*/
				Res.longitude = Long;
				Res.latitude = Lat;
				update_all(Res,now);

 				fflush(stdout);
			}

		}while(  --MaxPackPerChannel > 0 );/* Until we read enough */
		

	}/*  while  still sniffing */

	closePacket(RawSock);
	shutCard();
	pthread_exit(NULL);
}




#ifdef PRE_0_14 /* New version is in if_wlan-14.c */
/*
 * Check if data is avaliable 
 * Grab a packet from Interface
 */
int getPacket(unsigned char *buf,int maxlen,int timeout) {
	fd_set rs;
	int r;
	struct timeval tm;
	struct timeval *ptm;
	int  n;

	FD_ZERO(&rs);
	FD_SET(RawSock,&rs);
	FD_SET(0,&rs); /* Add stdin to it aswell */
	if (timeout>=0) {
		tm.tv_sec=timeout/1000;
		tm.tv_usec=(timeout%1000)*1000;
		ptm=&tm;
	} else {
		ptm=NULL;
	}
	r=select(RawSock+1,&rs,NULL,NULL,ptm);

	if (r<0){ 
		perror("Select failed");  
		return(-1);
	}
	if (r==0) {
		if (timeout>=0){ 
			/*printf("Timeout\n");*/  
			return(0);
		}
	}

	if( FD_ISSET(0,&rs) ){
		if( read( 0,&n,1) == 1 && ( n == 'Q' || n == 'q')){
			stop_sniffing =1;
			return 0;
		}
	}
	if (FD_ISSET(RawSock,&rs)) {
		r=recv(RawSock,buf,maxlen,0);

		if (r<0){ 
			printf(" recv failed\n"); 
			return(-1);
		}
	    return(r);
	  }

	  return(0); /* Noting to read */
}
#endif



