/***************************************************************************
                          ps_database.c  -  description
                             -------------------
    begin                : Sun Jun 30 2002
    copyright            : (C) 2002 by Jan Fernquist, Florian Boor
    email                : 
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <string.h>
#include "ps_database.h"
#include "psgui.h"
#include "prismstumbler.h"

#define SQLITE_OK           0   /* Successful result */
#define DB_DIR				".pstumbler"
#define DATABASENAME		"prismstumblerdb"
#define TABLE_NETWORKS		"networks"
// change pcrypt -> psum
#define SQL_CREATE_TABLE	"create table networks(bssid char(32), ssid char(40), first timestamp, last timestamp, type char(20), ip_range char(32), wep int, dhcp int, ap char(16), pdata long, pcrypt long, pint long, wep_key char(48), longitude float, latitude float, channel int, signal int, ishidden int,primary key(bssid))"
#define SQL_CHECK_TABLE		"select * from networks;"
#define SQL_GETALL			"select ssid,channel,bssid,signal from networks;"
#define SQL_SELECTBY_BSSID	"select * from networks where bssid = '%q';"
#define SQL_SELECTPOSBY_BSSID	"select ssid,bssid,longitude,latitude from networks where bssid = '%q';"
#define SQL_ADD_NETY		"insert into networks values('%q','%q',%i,%i,'%q','%q',%i,%i,'%q',%i,%i,%i,'%q',%f,%f);"
#define SQL_ADD_NET			"insert into networks (bssid,ssid,first,last,type,ip_range,wep,dhcp,ap,pdata,pcrypt,pint,wep_key,longitude,latitude,channel,signal,ishidden) values('%q','%q',%i,%i,'%q','%q',%i,%i,'%q',%ld,%i,%i,'%q',%f,%f,%i,%i,%i);"
#define SQL_SETUP_DB		"PRAGMA empty_result_callbacks = ON;"
#define SQL_UPDATE_NET		"update networks set ssid='%q',last=%i,wep=%i,dhcp=%i,ap='%q',pdata=%ld,pcrypt=%ld,pint=%ld,longitude=%f,latitude=%f,channel=%i, signal=%i,ishidden=%i, ip_range='%q' where bssid = '%q';"
#define SQL_DELETEBY_BSSID	"delete from networks where bssid = '%q' and channel = %i;"

//#define DEBUG

static sqlite* psdb = NULL;
char **myresult;
pthread_mutex_t db_mutex = PTHREAD_MUTEX_INITIALIZER;

// forward declarations
int getall_networks(sqlite_callback fCallback);
psnetinfo_t get_network(char* my_bssid);
int save_network(psnetinfo_t my_net);
int check_table();
int init_db();
int create_table();
void close_db();
void update_all(ScanResult_t aresult, time_t t_when);



inline void lock_db()
{
//	printf("lock db\n");
	pthread_mutex_lock(&db_mutex);
}


inline void unlock_db()
{
//	printf("unlock db\n");
	pthread_mutex_unlock(&db_mutex);
}


int deletenetwork(char* my_bssid,int chan)
{
	char* errmsg = NULL;
	int retval;
	int nrow = 0;
	int ncolumn = 0;
	lock_db();
	retval = sqlite_get_table_printf(psdb,SQL_DELETEBY_BSSID,&myresult,&nrow,&ncolumn,&errmsg,my_bssid,chan);
	unlock_db();
	
	if (retval){
		printf("Error delete: %s\n",errmsg);
		free(errmsg);
		return -1;
	}
	return 0;
}


void update_all(ScanResult_t aresult, time_t t_when)
{
	// add other updates here
	update_netlist(&aresult,t_when);
}


int getnetworkpos(char* my_bssid,sqlite_callback fCallback)
{
	char* errmsg = NULL;
	int retval;

	retval = sqlite_exec_printf(psdb,SQL_SELECTPOSBY_BSSID,fCallback,NULL,&errmsg,my_bssid);
	
	if (retval){
		printf("Error get networkpos: %s\n",errmsg);
		free(errmsg);
		return -1;
	}
	return 0;
}


int getall_networks(sqlite_callback fCallback)
{
	char* errmsg = NULL;
	int retval;

	retval = sqlite_exec(psdb,SQL_GETALL,fCallback,NULL,&errmsg);
	
	if (retval){
		printf("Error get networks: %s\n",errmsg);
		free(errmsg);
		return -1;
	}
	return 0;
}



psnetinfo_t get_network(char* my_bssid)
{
	char* errmsg = NULL;
	int nrow = 0;
	int ncolumn = 0;
	psnetinfo_t my_net;

	memset(&my_net,0,sizeof(psnetinfo_t));
	if (psdb)
	{
		#ifdef DEBUG
		printf("fetching network...\n");
		#endif
		lock_db();
		sqlite_get_table_printf(psdb,SQL_SELECTBY_BSSID,&myresult,&nrow,&ncolumn,&errmsg,my_bssid);
		unlock_db();

		if (nrow > 0) // we found this net in the database
		{
			// get data
			//HACK
			ncolumn-=1;
			strcpy(my_net.bssid,myresult[ncolumn+1]);
			strcpy(my_net.ssid,myresult[ncolumn+2]);
			my_net.first = atol(myresult[ncolumn+3]);
			my_net.last = atol(myresult[ncolumn+4]);

			strcpy(my_net.type,myresult[ncolumn+5]);
			strcpy(my_net.ip_range,myresult[ncolumn+6]);

			my_net.wep = atol(myresult[ncolumn+7]);
			my_net.dhcp = atol(myresult[ncolumn+8]);
			strcpy(my_net.ap,myresult[ncolumn+9]);
			my_net.pdata = atol(myresult[ncolumn+10]);
			my_net.psum = atol(myresult[ncolumn+11]);
			my_net.pint = atol(myresult[ncolumn+12]);
			strcpy(my_net.wep_key,myresult[ncolumn+13]);
			my_net.longitude = atof(myresult[ncolumn+14]);
			my_net.latitude = atof(myresult[ncolumn+15]);
			my_net.channel = atol(myresult[ncolumn+16]);
			my_net.maxsiglevel = atol(myresult[ncolumn+17]);
			my_net.ishidden = atol(myresult[ncolumn+18]);
			#ifdef DEBUG
			printf("found: %i c: %i\n",nrow,ncolumn);
			printf("read: %s\n",my_net.bssid);
			#endif
		}

		if (errmsg){
			printf("Error fetch network: %s\n",errmsg);
			free(errmsg);
		}
	}
	return my_net;
}


int save_network(psnetinfo_t my_net)
{
	char* errmsg = NULL;
	int nrow = 0;
	int ncolumn = 0;
	int ret;

	if (psdb)
	{
		#ifdef DEBUG
		printf("saving network...");
		#endif
		lock_db();
		sqlite_get_table_printf(psdb,SQL_SELECTBY_BSSID,&myresult,&nrow,&ncolumn,&errmsg,my_net.bssid);
		if (nrow > 0) // we found this net in the database
		{
			#ifdef DEBUG
			printf("updating\n");
			#endif
			// update network
			ret = sqlite_get_table_printf(psdb,SQL_UPDATE_NET,&myresult,&nrow,&ncolumn,&errmsg,
					my_net.ssid,my_net.last,my_net.wep,my_net.dhcp,my_net.ap,my_net.pdata,my_net.psum,
					my_net.pint,my_net.longitude,my_net.latitude,my_net.channel,my_net.maxsiglevel,my_net.ishidden,
					my_net.ip_range,my_net.bssid);
			if (ret){
				printf("Error db update: %s\n",errmsg);
				free(errmsg);
				unlock_db();
				return -1;
			}
		}
		else // new network
		{
			#ifdef DEBUG
			printf("adding\n");
			#endif
			// first = last - caution on c&p
			ret = sqlite_get_table_printf(psdb,SQL_ADD_NET,&myresult,&nrow,&ncolumn,&errmsg,my_net.bssid,my_net.ssid,my_net.last, my_net.last, my_net.type, my_net.ip_range, my_net.wep, my_net.dhcp, my_net.ap, my_net.pdata, my_net.psum, my_net.pint, my_net.wep_key, my_net.longitude, my_net.latitude, my_net.channel,my_net.maxsiglevel,my_net.ishidden);
			if (ret){
				printf("Error db adding network: %s\n",errmsg);
				free(errmsg);
				unlock_db();
				return -1;
			}
		}
		unlock_db();
	}
	return 0;
}


int check_table()
{
	char* errmsg = NULL;
	int nrow, ncolumn;

	if (!psdb) return -1;

	lock_db();
	sqlite_get_table(psdb,SQL_CHECK_TABLE,&myresult,&nrow,&ncolumn,&errmsg);
	unlock_db();
	if (errmsg){
		printf("Error db check: %s\n",errmsg);
		free(errmsg);
		return -1;
	}
    return 0;
}


int init_db()
{
	char* errmsg = NULL;
	int nrow, ncolumn;
    char* homedir;
	char* envtmp;
	DIR* dp;
	myresult = (char**)malloc(5);

	envtmp = getenv("HOME");
	if (!envtmp) return -1;

	homedir=malloc(strlen(envtmp)+1);
	homedir=strcpy(homedir,envtmp);
	
	homedir = realloc(homedir,strlen(homedir)+strlen(DB_DIR)+1);
	homedir = strcat(homedir,"/");
	homedir = strcat(homedir,DB_DIR);
	
	if ((dp = opendir(homedir)) == NULL)
	{
		if (mkdir(homedir,S_IRUSR | S_IWUSR | S_IXUSR)) return -1;
	}
	else closedir(dp);
	
	homedir = realloc(homedir,strlen(homedir)+strlen(DATABASENAME)+1);
	homedir = strcat(homedir,"/");
	homedir = strcat(homedir,DATABASENAME);
		
	lock_db();
	psdb = sqlite_open(homedir, 666, &errmsg);
	unlock_db();

	free(homedir);

	if (errmsg){
		printf("Error init db: %s\n",errmsg);
		free(errmsg);
		return -1;
	}
	lock_db();
	sqlite_get_table(psdb,SQL_SETUP_DB,&myresult,&nrow,&ncolumn,&errmsg);
	unlock_db();
	if (errmsg){
		printf("Error db setup: %s\n",errmsg);
		free(errmsg);
		return -1;
	}
	return 0;
}


int create_table()
{
	char* errmsg = NULL;
	int nrow, ncolumn;
	int retval;
	psnetinfo_t* new_net;

	if (!psdb) return -1;

	lock_db();
	retval = sqlite_get_table(psdb,SQL_CREATE_TABLE,&myresult,&nrow,&ncolumn,&errmsg);
	unlock_db();
	
	if (retval){
		printf("Error creating db: %s\n",errmsg);
		free(errmsg);
		return -1;
	}
	
	new_net = malloc(sizeof(psnetinfo_t));
    memset(new_net,0,sizeof(psnetinfo_t));
	
	sprintf(new_net->ssid,"%s","this could be yours");
	sprintf(new_net->bssid,"%s","00:00:00:00:00:00");

	save_network(*new_net);

	free(new_net);
	return 0;
}


void close_db()
{
	lock_db();
	sqlite_close(psdb);
	unlock_db();
}

