#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include <pthread.h>

#include "psgui.h"
#include "interface.h"
#include "support.h"
#include "ps_database.h"
#include "ps_resolv.h"
#include "helper.h"
#include "analyzer.h"

#define COLORSTEP 1000
#define UPDATE_INTERVAL 1200

pthread_mutex_t gui_mutex = PTHREAD_MUTEX_INITIALIZER;

extern int SingleChannel;
extern unsigned int stop_sniffing;
extern FILE * WPFile;
extern FILE * OutFile;
gchar logbuf[120];

static int actual_rows = -1;
static int interval_maxsiglevel = 0;
static gushort colorlevels[1024]; // FIXME: dynamic!

static GtkCList* gclWlans;
static GtkProgressBar* gpbSignal;
static GtkText* txLog;

static guint active_row = -1;
static guint last_row = 0;
void stop_signal ();


/*
 * Signal Handler
 */
void stop_signal () {
  fprintf (stderr,"Received CTRL-C - sniffing aborted\n");
  on_fMain_destroy(NULL,NULL);
}


void printlog(gchar* str)
{
	gtk_text_insert(txLog,NULL,NULL,NULL,str,-1);
}


inline void lock_gui()
{
//	don't need this with standard gtk
//	pthread_mutex_lock(&gui_mutex);
}


inline void unlock_gui()
{
//	don't need this with standard gtk
//	pthread_mutex_unlock(&gui_mutex);
}


// do in timer intervals
gint do_timer_test()
{
	GtkNotebook* widget;
	widget = (GtkNotebook*)lookup_widget((GtkWidget*)gclWlans,"notebook1");
	
	// if page2 active - update there
	if (gtk_notebook_get_current_page(widget) == 1) 
		gtk_clist_select_row(gclWlans,active_row,0);
	return TRUE;
}

gint do_timer_update()
{
	int rpos;
	psnetinfo_t* anet;
	gchar tmpstr[80];
	GdkColor wcolor;
	GtkWidget* togglebutton;

	wcolor.red = 65535;
	wcolor.blue = 65535;
	// update list and database
	lock_gui();

	/*	If we find a mess of nets, this could be a trick... check if it is a clever one
		After tests bail out here and keep list clean.
	*/
	if (newnet_count > FAKEAP_LEVEL)
	{
		newnet_count = 0;
		FAKEAP_FILTER = TRUE;
		togglebutton = lookup_widget((GtkWidget*)gclWlans,"cbFakeAP");
		gtk_toggle_button_set_active((GtkToggleButton *)togglebutton,TRUE);
		// clean list here
	}

	newnet_count = 0;
	gtk_clist_freeze(gclWlans);	

	for (rpos=0; rpos < gclWlans->rows ; rpos++)
	{
        anet = gtk_clist_get_row_data(gclWlans,rpos);

		if (anet->isvalid) // flag shows if data from this net was received
		{
			anet->isvalid = FALSE;
			save_network(*anet);
			if (check_str_content(anet->ssid)) gtk_clist_set_text(gclWlans,rpos,0,anet->ssid);
			sprintf(tmpstr,"%i",anet->channel);	
			gtk_clist_set_text(gclWlans,rpos,1,tmpstr);	
			sprintf(tmpstr,"%i",anet->cursiglevel);
			anet->cursiglevel = 0;	
			gtk_clist_set_text(gclWlans,rpos,3,tmpstr);
			wcolor.green = 0;
			colorlevels[rpos] = 0;
			gtk_clist_set_background(gclWlans,rpos,&wcolor);
		}
	}

	gtk_clist_thaw(gclWlans);	

	// update levelmeter
	gtk_progress_set_value((GtkProgress*)gpbSignal,(gfloat)interval_maxsiglevel);
	interval_maxsiglevel = 0;
	unlock_gui();

	// clear and recreate waypoint list
 	// update colors here (in future)
	return TRUE;
}


int waypointsAddNet(void *pArg, int argc, char **argv, char **columnNames)
{
	if ((WPFile) && (argc>3))
	{
		if (strlen(argv[0]) > 0) fprintf(WPFile,"%s \t%s \t%s\n",argv[0],argv[2],argv[3]);
			else fprintf(WPFile,"%s \t%s \t%s\n",argv[1],argv[2],argv[3]);
	}
	return 0;
}


void update_set_idle()
{
  // currently not used
}


void update_colors()
{
	int rpos = 0;
	GdkColor wcolor;

	lock_gui();

	wcolor.red = 65535;
	wcolor.blue = 65535;

	gtk_clist_freeze(gclWlans);	
	for (rpos = 0;rpos <= actual_rows; rpos++)
	{
		if (colorlevels[rpos] < 65535)
		{
			wcolor.green = colorlevels[rpos];
			if (colorlevels[rpos] > (65535-COLORSTEP))
				wcolor.green = 65535;
			else
			{	
				wcolor.green = colorlevels[rpos];
				wcolor.green += ((65535-colorlevels[rpos]) / 8);
			}	
			colorlevels[rpos] = wcolor.green;
			gtk_clist_set_background(gclWlans,rpos,&wcolor);
		}
	}
	gtk_clist_thaw(gclWlans);
	unlock_gui();
	
}


void update_netlist(ScanResult_t* ares, time_t t_when)
{
	int rpos = 0;
	int res;
    gchar* a_bss;
	gchar *tmpvec[4];
	GdkColor wcolor;
	int found_net = FALSE;
	psnetinfo_t *new_net;
	wcolor.red = 65535;
	wcolor.blue = 65535;
	lock_gui();

	// update signal strength
	if (ares->Signal > interval_maxsiglevel) interval_maxsiglevel = ares->Signal;

	for (rpos = 0;rpos <= actual_rows; rpos++)
	{
		res = gtk_clist_get_text(gclWlans,rpos,2,&a_bss);
		if (res)
		{
			// update network data
			if (!strncmp(a_bss,ares->BssId,17))
			{

                // lets hack a data-transfer
				found_net = TRUE;
				new_net = gtk_clist_get_row_data(gclWlans,rpos);
				new_net->isvalid = TRUE;

				// in some cases so we get a hidden network ssid
				if (check_str_content(ares->SSID))
				{
				    if (!check_str_content(new_net->ssid)) new_net->ishidden = TRUE;
					strcpy(new_net->ssid,ares->SSID);
				}
				
				if (ares->isAp > (atoi(new_net->ap))) sprintf(new_net->ap,"%i",ares->isAp);
				new_net->last = t_when;
				new_net->psum += 1;

				new_net->pdata += ares->isData;
				new_net->pint += ares->hasIntIV;
				new_net->dhcp += ares->dhcp;

				sprintf(new_net->type,"%.4X",ares->FrameType);

				if (ares->hasWep > new_net->wep) new_net->wep = ares->hasWep;
				new_net->channel = ares->Channel;		

				if (ares->Signal > new_net->cursiglevel) new_net->cursiglevel = ares->Signal;

				if (new_net->maxsiglevel <= ares->Signal) // save only strongest siglevel+position
				{
					new_net->longitude = ares->longitude;
					new_net->latitude = ares->latitude;
					new_net->maxsiglevel = ares->Signal;
				}
				else
					if (!(new_net->longitude) && (ares->longitude)) // update position if no saved pos and pos now available
					{
						new_net->longitude = ares->longitude;
						new_net->latitude = ares->latitude;
					}

				if ((ares->subnet[0])!=0) {
				 	new_net->ip_range[0]=ares->subnet[0];
				 	new_net->ip_range[1]=ares->subnet[1];
				 	new_net->ip_range[2]=ares->subnet[2];
				 	new_net->ip_range[3]=ares->subnet[3];
				}
				unlock_gui();

				return;
			}		
		}     	
	}
	
	// we have a new contact
	if ((!found_net) && ((!FAKEAP_FILTER) || (FAKEAP_FILTER && ares->isData)))
	{

		newnet_count++;

		// new row
		tmpvec[0] = malloc(80*sizeof(gchar));
		tmpvec[1] = malloc(80*sizeof(gchar));
		tmpvec[2] = malloc(80*sizeof(gchar));
		tmpvec[3] = malloc(80*sizeof(gchar));

		strcpy(tmpvec[0],ares->SSID);
		sprintf(tmpvec[1],"%i",ares->Channel);	
		strcpy(tmpvec[2],ares->BssId);
		sprintf(tmpvec[3],"%i",ares->Signal);	

		rpos = gtk_clist_prepend(gclWlans,tmpvec);
		actual_rows++;//(gclWlans->rows-1); // saves max index!
		wcolor.green = 0;

		// move colors...
		for (res=(actual_rows+1);res >= 0;res--)
		{
			colorlevels[res+1]=colorlevels[res];
		}		
		colorlevels[rpos] = 0;
		
		gtk_clist_set_background(gclWlans,rpos,&wcolor);

		free(tmpvec[0]);
		free(tmpvec[1]);
		free(tmpvec[2]);
		free(tmpvec[3]);

		// add data
		new_net = malloc(sizeof(psnetinfo_t));

		memset(new_net,0,sizeof(psnetinfo_t));
		new_net->ishidden = FALSE;
		strcpy(new_net->ssid,ares->SSID);
		strcpy(new_net->bssid,ares->BssId);
				
		new_net->last = t_when;
		new_net->first = t_when;
		new_net->pint = 0;
		new_net->dhcp = 0;
		new_net->psum = 1;
		if (ares->isData) new_net->pdata = 1;
		if (ares->hasIntIV) new_net->pint = 1;
			
		sprintf(new_net->wep_key,"<unknown>");

		sprintf(new_net->type,"%.4X",ares->FrameType);
		sprintf(new_net->ap,"%i",ares->isAp);

		new_net->wep = ares->hasWep;
		new_net->channel = ares->Channel;		

		new_net->longitude = ares->longitude;
		new_net->latitude = ares->latitude;
		new_net->maxsiglevel = ares->Signal;
		new_net->cursiglevel = ares->Signal;

		if (WPFile)
		{
			getnetworkpos(ares->BssId,waypointsAddNet);
		}
		if ((ares->subnet[3])==0) {
			memcpy(new_net->ip_range,ares->subnet,3);	
		}
		else
			memset(&(new_net->ip_range),' ',4);					
		gtk_clist_set_row_data(gclWlans,rpos,(gpointer)new_net);
		
		sprintf(logbuf,"Found new net: \n %s %s \n",new_net->ssid,new_net->bssid);
		printlog(logbuf);
	}
	unlock_gui();
}


int gridAddNet(void *pArg, int argc, char **argv, char **columnNames)
{
    psnetinfo_t *new_net;
	psnetinfo_t anet;
	
	anet.isvalid = 5777;
	lock_gui();

	// add data
	if (argv && argv[2])
	{
		anet = get_network(argv[2]);
		if (anet.isvalid != 5777) {
			gtk_clist_prepend(gclWlans,argv);
			actual_rows++;
			anet.isvalid = FALSE; // we received no data from this net
			new_net = malloc(sizeof(psnetinfo_t));
			memcpy(new_net,&anet,sizeof(psnetinfo_t));
			gtk_clist_set_row_data(gclWlans,0,(gpointer)new_net);
			if (WPFile)
			{
				if (argv && argv[2]) getnetworkpos(argv[2],waypointsAddNet);
			}
		}
		else
			fprintf(stderr,"urks - got empty net :-/");
	}

	unlock_gui();
	
	return 0;
}


void clear_list()
{
	lock_gui();
	gtk_clist_clear(gclWlans);
	unlock_gui();
}


void
on_fMain_destroy                       (GtkObject       *object,
                                        gpointer         user_data)
{
    stop_sniffing = TRUE;
	sleep(1);
	if(OutFile){
		fclose(OutFile);
	}

	if(WPFile){
		fclose(WPFile);
	}

	pthread_exit(NULL);
	close_db();
	gtk_exit(0);
}


void
on_fMain_show                          (GtkWidget       *widget,
                                    gpointer         user_data)
{
	GtkEntry* eChannel;
	gchar* cstr;	

	// we do the rest of initialisation here...
	memset(colorlevels,65535,1024*sizeof(gushort));

	init_db();
	if (check_table() != 0)
	{
		if (create_table() != 0){
			fprintf (stderr,"Unable to create database - exiting!\n");
			pthread_exit(NULL);
			gtk_exit(-1);
		}
	}

	signal (SIGINT, stop_signal);
	signal (SIGTERM, stop_signal);

	gclWlans = (GtkCList*)lookup_widget(widget,"gclWlans");
	gpbSignal = (GtkProgressBar*)lookup_widget(widget,"pg_signal");
	txLog = (GtkText*)lookup_widget(widget,"tx_log");

	if (SingleChannel)
	{
		cstr=malloc(10*sizeof(gchar));
		eChannel = (GtkEntry*)lookup_widget((GtkWidget*)gclWlans,"eSChannel");
		sprintf(cstr,"%i",SingleChannel);
		gtk_entry_set_text(eChannel,cstr);
		free(cstr);
		eChannel = (GtkEntry*)lookup_widget((GtkWidget*)gclWlans,"rbCSpecific");
		gtk_toggle_button_set_state((GtkToggleButton*)eChannel,TRUE);
	}

	gtk_clist_freeze(gclWlans);
	
	gtk_window_set_title (GTK_WINDOW (lookup_widget(widget,"fMain")), VERSIONSTR);
	gtk_text_insert(txLog,NULL,NULL,NULL,"eventlog opened\n",-1);

	getall_networks(gridAddNet);

	// setup periodically update
	gtk_timeout_add(UPDATE_INTERVAL,(GtkFunction)do_timer_update,NULL);
	gtk_timeout_add(UPDATE_INTERVAL,(GtkFunction)do_timer_test,NULL);

 	gtk_clist_thaw(gclWlans);
	gtk_clist_select_row(gclWlans,0,0);
}


void
on_gclWlans_realize                    (GtkWidget       *widget,
                                        gpointer         user_data)
{
}


void
on_gclWlans_select_row                 (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
   gchar* selected_bssid;
	psnetinfo_t selected_net;
	GtkLabel* lCur;
	GtkCList* gclDetail;
	gchar tmpstr[80];
	gchar* tmpstr2;
	gchar *tmpvec[2];
	gint res;
	guint8 binbssid[8];
    unsigned int b0,b1,b2;
 
	lock_gui();

	last_row = active_row;
	active_row = row;

	gclDetail = (GtkCList*)lookup_widget((GtkWidget*)clist,"gclDetail");

	gtk_clist_clear(gclDetail);

	res = gtk_clist_get_text(gclWlans,row,2,&selected_bssid);

	if (res)
	{
		selected_net = get_network(selected_bssid);
        tmpvec[0] = malloc(80*sizeof(gchar));
        tmpvec[1] = malloc(80*sizeof(gchar));
		gtk_clist_freeze(gclDetail);
		strcpy(tmpvec[0],"BSSID");
		strcpy(tmpvec[1],selected_net.bssid);
		gtk_clist_append(gclDetail,tmpvec);
		strcpy(tmpvec[0],"SSID");
		strcpy(tmpvec[1],selected_net.ssid);
		gtk_clist_append(gclDetail,tmpvec);	

		strcpy(tmpvec[0],"Channel");
		sprintf(tmpvec[1],"%i",selected_net.channel);
		gtk_clist_append(gclDetail,tmpvec);
		strcpy(tmpvec[0],"max signal level");
		sprintf(tmpvec[1],"%i",selected_net.maxsiglevel);
		gtk_clist_append(gclDetail,tmpvec);

		strcpy(tmpvec[0],"first seen");
 		memset(tmpstr,0,80);
		tmpstr2 = ctime_r(&selected_net.first,tmpstr);
		strncpy(tmpvec[1],tmpstr2,strlen(tmpstr2));
		tmpvec[1][strlen(tmpstr2)-1]=0;
		gtk_clist_append(gclDetail,tmpvec);

		strcpy(tmpvec[0],"last seen");
		tmpstr2 = ctime_r(&selected_net.last,tmpstr);
		strncpy(tmpvec[1],tmpstr2,strlen(tmpstr2));
		tmpvec[1][strlen(tmpstr2)-1]=0;
		gtk_clist_append(gclDetail,tmpvec);

		strcpy(tmpvec[0],"Frametype");
		strcpy(tmpvec[1],selected_net.type);
		gtk_clist_append(gclDetail,tmpvec);

		strcpy(tmpvec[0],"Accesspoint");
		switch (atoi(selected_net.ap))
		{
			case AP_NONE:	sprintf(tmpvec[1],"%s","none");
				break;
			case AP_ONE:	sprintf(tmpvec[1],"%s","one");
				break;
			case AP_MORE:	sprintf(tmpvec[1],"%s","> one");
				break;
			case AP_PROBE:	sprintf(tmpvec[1],"%s","probe for");
				break;
			default:  sprintf(tmpvec[1],"%s","<unknown>");
				break;
		}
		gtk_clist_append(gclDetail,tmpvec);
	
		strcpy(tmpvec[0],"Subnet");
		sprintf(tmpvec[1],"%i.%i.%i.%i",selected_net.ip_range[0],selected_net.ip_range[1],selected_net.ip_range[2],selected_net.ip_range[3]);

		gtk_clist_append(gclDetail,tmpvec);

		strcpy(tmpvec[0],"hidden");
		sprintf(tmpvec[1],"%i",selected_net.ishidden);
		gtk_clist_append(gclDetail,tmpvec);
		strcpy(tmpvec[0],"WEP");
		sprintf(tmpvec[1],"%i",selected_net.wep);
		gtk_clist_append(gclDetail,tmpvec);
		strcpy(tmpvec[0],"DHCP");
		sprintf(tmpvec[1],"%i",selected_net.dhcp);
		gtk_clist_append(gclDetail,tmpvec);

        sscanf(selected_net.bssid, "%x:%x:%x", &b0, &b1, &b2);
		strcpy(tmpvec[0],"Manufacturer");
		binbssid[0] = b0;
		binbssid[1] = b1;
		binbssid[2] = b2;
		sprintf(tmpvec[1],"%s",get_manuf_name(binbssid));
		gtk_clist_append(gclDetail,tmpvec);

		lCur = (GtkLabel*)lookup_widget((GtkWidget*)clist,"lpsum");
		sprintf(tmpstr,"%ld",selected_net.psum);
		gtk_label_set_text(lCur,tmpstr);
		lCur = (GtkLabel*)lookup_widget((GtkWidget*)clist,"lpdata");
		sprintf(tmpstr,"%ld",selected_net.pdata);
		gtk_label_set_text(lCur,tmpstr);
		lCur = (GtkLabel*)lookup_widget((GtkWidget*)clist,"lpiiv");
		sprintf(tmpstr,"%ld",selected_net.pint);
		gtk_label_set_text(lCur,tmpstr);
		lCur = (GtkLabel*)lookup_widget((GtkWidget*)clist,"lwepkey");
		sprintf(tmpstr,"%s",selected_net.wep_key);
		gtk_label_set_text(lCur,tmpstr);
		lCur = (GtkLabel*)lookup_widget((GtkWidget*)clist,"llong");
		sprintf(tmpstr,"%f",selected_net.longitude);
		gtk_label_set_text(lCur,tmpstr);
		lCur = (GtkLabel*)lookup_widget((GtkWidget*)clist,"llat");
		sprintf(tmpstr,"%f",selected_net.latitude);
		gtk_label_set_text(lCur,tmpstr);

		gtk_clist_thaw(gclDetail);

		free(tmpvec[0]);
		free(tmpvec[1]);
	}
	unlock_gui();

}



void
on_notebook1_switch_page               (GtkNotebook     *notebook,
                                        GtkNotebookPage *page,
                                        gint             page_num,
                                        gpointer         user_data)
{
	if (active_row >= 0)
		gtk_clist_select_row(gclWlans,active_row,0);
	else
		gtk_clist_select_row(gclWlans,last_row,0);
}


gboolean
on_gclWlans_button_press_event         (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkMenu *menu;
  GdkEventButton *event_button;

  menu = GTK_MENU(create_menu1());
  if (event->type == GDK_BUTTON_PRESS)
    {
      event_button = (GdkEventButton *) event;
      if (event_button->button == 3)
		{
		  gtk_menu_popup (menu, NULL, NULL, NULL, NULL,
			  event_button->button, event_button->time);
		  return TRUE;
		}
    }
    if ( event->type==GDK_2BUTTON_PRESS || event->type==GDK_3BUTTON_PRESS) 
	{
		gtk_notebook_set_page((GtkNotebook*)lookup_widget((GtkWidget*)gclWlans,"notebook1"),1);
	}
	return FALSE;
}


void
on_delete1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gchar* dbssid;
	gchar* dchan;
	int a;

	if (active_row >= 0)
	{
		lock_gui();
		gtk_clist_get_text(gclWlans,active_row,2,&dbssid);
		gtk_clist_get_text(gclWlans,active_row,1,&dchan);
		if (!deletenetwork(dbssid,atoi(dchan)))
		{
			free(gtk_clist_get_row_data(gclWlans,active_row));
			gtk_clist_remove(gclWlans,active_row);
			actual_rows--;
			for (a=active_row;a<=actual_rows-1;a++)
			{
				colorlevels[a]=colorlevels[a+1];
			}		

			active_row=-1;	
		}
		unlock_gui();
	}
}


void
on_view_detail1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gtk_notebook_set_page((GtkNotebook*)lookup_widget((GtkWidget*)gclWlans,"notebook1"),1);
}


void
on_gclWlans_unselect_row               (GtkCList        *clist,
                                        gint             row,
                                        gint             column,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	last_row = row;
	active_row=-1;
}


void
on_cbNight_toggled                     (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

	GtkStyle* astyle;
	
 	static GdkColor blue = {0,0,0x0000,0x8000};
 	static GdkColor green = {0,0,0x8000,0x0000};
 	static GdkColor white = {0,0xffff,0xffff,0xffff};
 	static GdkColor black = {0,0x0000,0x0000,0x0000};

	astyle=gtk_widget_get_style((GtkWidget*)gclWlans);
	if (gtk_toggle_button_get_active(togglebutton))
	{
		astyle->rc_style->base[GTK_STATE_NORMAL] = blue;
 		astyle->rc_style->text[GTK_STATE_NORMAL] = white;
 		astyle->rc_style->fg[GTK_STATE_NORMAL] = white;
		astyle->rc_style->color_flags[GTK_STATE_NORMAL] |= GTK_RC_FG;
		astyle->rc_style->color_flags[GTK_STATE_NORMAL] |= GTK_RC_TEXT;
		astyle->rc_style->color_flags[GTK_STATE_NORMAL] |= GTK_RC_BASE;

		astyle->rc_style->bg[GTK_STATE_SELECTED] = green;
		astyle->rc_style->color_flags[GTK_STATE_SELECTED] |= GTK_RC_BG;
	}
	else
	{
 		astyle->rc_style->base[GTK_STATE_NORMAL] = white;
 		astyle->rc_style->text[GTK_STATE_NORMAL] = black;
 		astyle->rc_style->fg[GTK_STATE_NORMAL] = black;
		astyle->rc_style->bg[GTK_STATE_SELECTED] = blue;
    }

	gtk_widget_modify_style(GTK_WIDGET(gclWlans),astyle->rc_style);
	gtk_widget_ensure_style(GTK_WIDGET(gclWlans));

	astyle=gtk_widget_get_style(GTK_WIDGET(lookup_widget(GTK_WIDGET(gclWlans),"gclDetail")));
	if (gtk_toggle_button_get_active(togglebutton))
	{
 		astyle->rc_style->base[GTK_STATE_NORMAL] = blue;
 		astyle->rc_style->text[GTK_STATE_NORMAL] = white;
 		astyle->rc_style->fg[GTK_STATE_NORMAL] = white;
		astyle->rc_style->color_flags[GTK_STATE_NORMAL] |= GTK_RC_FG;
		astyle->rc_style->color_flags[GTK_STATE_NORMAL] |= GTK_RC_TEXT;
		astyle->rc_style->color_flags[GTK_STATE_NORMAL] |= GTK_RC_BASE;

		astyle->rc_style->bg[GTK_STATE_SELECTED] = green;
		astyle->rc_style->color_flags[GTK_STATE_SELECTED] |= GTK_RC_BG;
    }
	else
	{
 		astyle->rc_style->base[GTK_STATE_NORMAL] = white;
 		astyle->rc_style->text[GTK_STATE_NORMAL] = black;
 		astyle->rc_style->fg[GTK_STATE_NORMAL] = black;
		astyle->rc_style->bg[GTK_STATE_SELECTED] = blue;
    }
	gtk_widget_modify_style(GTK_WIDGET(lookup_widget(GTK_WIDGET(gclWlans),"gclDetail")), astyle->rc_style);	
}



gboolean
on_fMain_key_press_event               (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	switch (event->keyval){
		case GDK_d:
		gtk_notebook_set_page((GtkNotebook*)lookup_widget((GtkWidget*)gclWlans,"notebook1"),1);
		return TRUE;
		break;
		case GDK_l:
		gtk_notebook_set_page((GtkNotebook*)lookup_widget((GtkWidget*)gclWlans,"notebook1"),0);
		return TRUE;
		break;
		case GDK_c:
		gtk_notebook_set_page((GtkNotebook*)lookup_widget((GtkWidget*)gclWlans,"notebook1"),2);
		return TRUE;
		break;
		case GDK_a:
		gtk_notebook_set_page((GtkNotebook*)lookup_widget((GtkWidget*)gclWlans,"notebook1"),3);
		return TRUE;
		break;
		case GDK_r:
		on_delete1_activate(NULL,NULL);
		return TRUE;
		break;
		case GDK_p:
		case GDK_Up:
		if (gtk_notebook_current_page((GtkNotebook*)lookup_widget((GtkWidget*)gclWlans,"notebook1")) == 1)
		{
			gtk_clist_select_row(gclWlans,active_row-1,0);
			return TRUE;		
		}
		else
			return FALSE;
		break;
		case GDK_n:
		case GDK_Down:
		if (gtk_notebook_current_page((GtkNotebook*)lookup_widget((GtkWidget*)gclWlans,"notebook1")) == 1)
		{
			gtk_clist_select_row(gclWlans,active_row+1,0);
			return TRUE;		
		}
		else
			return FALSE;
		break;
// Waiting for implementation
/*		case GDK_w:
		gtk_notebook_set_page((GtkNotebook*)lookup_widget((GtkWidget*)gclWlans,"notebook1"),4);
		return TRUE;
		break;
*/	}
	return FALSE;
}


void
on_rbCAll_toggled                      (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	SingleChannel = 0;
}


void
on_rbCSpecific_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if (gtk_toggle_button_get_active(togglebutton))
	{
		GtkEntry* eChannel;
		gchar* cstr;	

		eChannel = (GtkEntry*)lookup_widget((GtkWidget*)gclWlans,"eSChannel");

		cstr = gtk_editable_get_chars((GtkEditable*)eChannel,0,3);
		SingleChannel = atoi(cstr);

		if( SingleChannel < 1 || SingleChannel > 14 ){
			SingleChannel = 0;
			printf("Illegal channel!\n");
		}

		g_free(cstr);
	}
}


gboolean
on_eSChannel_focus_out_event           (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
	GtkWidget *togglebutton;
	togglebutton = lookup_widget((GtkWidget*)gclWlans,"rbCSpecific");
	gtk_toggle_button_set_active((GtkToggleButton *)togglebutton,TRUE);
	// sometimes we need force...
	on_rbCSpecific_toggled((GtkToggleButton *)togglebutton,NULL);
	return FALSE;
}


void
on_cbLogPackets_toggled                (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	LOG_PACKETS = gtk_toggle_button_get_active(togglebutton);
}


void
on_cbFakeAP_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	FAKEAP_FILTER = gtk_toggle_button_get_active(togglebutton);
}

